@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-embed.c 7712 2007-11-17 17:11:40Z xan $
+ *  $Id: ephy-embed.c 7836 2007-12-30 00:24:40Z xan $
  */
 
 #include "config.h"
@@ -676,81 +676,6 @@ ephy_embed_scroll_pixels (EphyEmbed *embed,
 	iface->scroll_pixels (embed, dx, dy);
 }
 
-/**
- * ephy_embed_shistory_n_items:
- * @embed: an #EphyEmbed
- *
- * Returns the number of items in @embed's history. In other words, returns the
- * number of pages @embed has visited.
- *
- * The number is upper-bound by Mozilla's browser.sessionhistory.max_entries
- * preference.
- *
- * Return value: the number of items in @embed's history
- **/
-int
-ephy_embed_shistory_n_items  (EphyEmbed *embed)
-{
-	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
-	return iface->shistory_n_items (embed);
-}
-
-/**
- * ephy_embed_shistory_get_nth:
- * @embed: an #EphyEmbed
- * @nth: index of the desired page in @embed's browser history
- * @is_relative: if %TRUE, add @embed's current history position to @nth
- * @url: returned value of the history entry's URL
- * @title: returned value of the history entry's title
- *
- * Fetches the @url and @title of the @nth item in @embed's session history.
- * If @is_relative is %TRUE, @nth is an offset from the browser's current
- * history position. For example, calling this function with @is_relative %TRUE
- * and @nth %0 will return the URL and title of the current page.
- **/
-void
-ephy_embed_shistory_get_nth (EphyEmbed *embed,
-			     int nth,
-			     gboolean is_relative,
-			     char **url,
-			     char **title)
-{
-	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
-	iface->shistory_get_nth (embed, nth, is_relative, url, title);
-}
-
-/**
- * ephy_embed_shistory_get_pos:
- * @embed: an #EphyEmbed
- *
- * Returns @embed's current position in its history. If the user never uses the
- * "Back" button, this number will be the same as the return value of
- * ephy_embed_shistory_n_items().
- *
- * Return value: @embed's current position in its history
- **/
-int
-ephy_embed_shistory_get_pos (EphyEmbed *embed)
-{
-	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
-	return iface->shistory_get_pos (embed);
-}
-
-/**
- * ephy_embed_shistory_go_nth:
- * @embed: an #EphyEmbed
- * @nth: desired history index
- *
- * Opens the webpage at location @nth in @embed's history.
- **/
-void
-ephy_embed_shistory_go_nth (EphyEmbed *embed,
-			    int nth)
-{
-	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
-	iface->shistory_go_nth (embed, nth);
-}
-
 /**
  * ephy_embed_shistory_copy:
  * @source: the #EphyEmbed to copy the history from
@@ -1138,7 +1063,7 @@ ephy_embed_get_loading_title (EphyEmbed *embed)
  * Returns whether the @embed's toplevel is visible or not. Used
  * mostly for popup visibility management.
  *
- * Return value; %TRUE if @embed's "visibility" property is set
+ * Return value: %TRUE if @embed's "visibility" property is set
  **/
 gboolean
 ephy_embed_get_visibility (EphyEmbed *embed)
@@ -1146,3 +1071,80 @@ ephy_embed_get_visibility (EphyEmbed *embed)
 	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
 	return iface->get_visibility (embed);
 }
+
+/**
+ * ephy_embed_get_backward_history:
+ * @embed: an #EphyEmbed
+ *
+ * Returns a #GList of #EphyHistoryItem compromising the
+ * history items preceding the current location.
+ *
+ * Return value: a #GList with the preceding history items
+ **/
+GList*
+ephy_embed_get_backward_history (EphyEmbed *embed)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
+	return iface->get_backward_history (embed);
+}
+
+/**
+ * ephy_embed_get_forward_history:
+ * @embed: an #EphyEmbed
+ *
+ * Returns a #GList of #EphyHistoryItem compromising the
+ * history items succeeding the current location.
+ *
+ * Return value: a #GList with the succeeding history items
+ **/
+GList*
+ephy_embed_get_forward_history (EphyEmbed *embed)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
+	return iface->get_forward_history (embed);
+}
+
+/**
+ * ephy_embed_get_previous_history_item:
+ * @embed: an #EphyEmbed
+ *
+ * Returns the preceding #EphyHistoryItem in the history list.
+ *
+ * Return value: the preceding #EphyHistoryItem
+ **/
+EphyHistoryItem*
+ephy_embed_get_previous_history_item (EphyEmbed *embed)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
+	return iface->get_previous_history_item (embed);
+}
+
+/**
+ * ephy_embed_get_next_history_item:
+ * @embed: an #EphyEmbed
+ *
+ * Returns the succeeding #EphyHistoryItem in the history list.
+ *
+ * Return value: the succeeding #EphyHistoryItem
+ **/
+EphyHistoryItem*
+ephy_embed_get_next_history_item (EphyEmbed *embed)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
+	return iface->get_next_history_item (embed);
+}
+
+/**
+ * ephy_embed_go_to_history_item:
+ * @embed: an #EphyEmbed
+ * @history_item: an #EphyHistoryItem
+ *
+ * Opens the webpage specified by @history_item in @embed's history.
+ *
+ **/
+void
+ephy_embed_go_to_history_item (EphyEmbed *embed, EphyHistoryItem *history_item)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
+	iface->go_to_history_item (embed, history_item);
+}
