@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-password-manager.h 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ephy-password-manager.h 7623 2007-11-04 23:35:28Z cosimoc $
  */
 
 #ifndef EPHY_PASSWORD_MANAGER_H
@@ -58,6 +58,7 @@ struct _EphyPasswordManagerIface
 				 EphyPasswordInfo *info);
 	void	(* remove)	(EphyPasswordManager *manager,
 				 EphyPasswordInfo *info);
+	void	(* remove_all)	(EphyPasswordManager *manager);
 	GList *	(* list)	(EphyPasswordManager *manager);
 };
 
@@ -83,6 +84,8 @@ void		ephy_password_manager_add_password	(EphyPasswordManager *manager,
 void		ephy_password_manager_remove_password	(EphyPasswordManager *manager,
 							 EphyPasswordInfo *info);
 
+void		ephy_password_manager_remove_all_passwords (EphyPasswordManager *manager);
+
 GList *		ephy_password_manager_list_passwords	(EphyPasswordManager *manager);
 
 G_END_DECLS
