@@ -38,7 +38,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp 7858 2008-01-13 20:42:01Z cosimoc $
+ * $Id: MozDownload.cpp 7888 2008-01-24 19:14:16Z cosimoc $
  */
 
 #include <xpcom-config.h>
@@ -400,8 +400,12 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 				rv = mDestination->GetSpec (aDest);
 				NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-				ephy_file_browse_to (aDest.get (), user_time);
+				GFile *dest;
 
+				dest = g_file_new_for_uri (dest);
+				ephy_file_browse_to (dest, user_time);
+
+				g_object_unref (dest);
 				g_strfreev (str);
 			}
 #endif /* HAVE_GECKO_1_9 */
