@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: ephy-bookmarks-editor.c 7719 2007-11-18 23:53:16Z cosimoc $
+ * $Id: ephy-bookmarks-editor.c 7791 2007-12-19 21:46:55Z cosimoc $
  */
 
 #include "config.h"
@@ -1280,6 +1280,25 @@ ephy_bookmarks_editor_node_activated_cb (GtkWidget *view,
 			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
+static void
+ephy_bookmarks_editor_node_middle_clicked_cb (GtkWidget *view,
+					      EphyNode *node,
+					      EphyBookmarksEditor *editor)
+{
+	EphyWindow *window;
+	const char *location;
+
+	window = EPHY_WINDOW (get_target_window (editor));
+
+	location = ephy_node_get_property_string
+		(node, EPHY_NODE_BMK_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
+
+	ephy_shell_new_tab (ephy_shell, window, NULL, location,
+			    EPHY_NEW_TAB_OPEN_PAGE |
+			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+}
+
 static void
 ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 {
@@ -1970,6 +1989,10 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 			  "node_activated",
 			  G_CALLBACK (ephy_bookmarks_editor_node_activated_cb),
 			  editor);
+	g_signal_connect (G_OBJECT (bm_view),
+			  "node_middle_clicked",
+			  G_CALLBACK (ephy_bookmarks_editor_node_middle_clicked_cb),
+			  editor);
 	g_signal_connect (G_OBJECT (bm_view),
 			  "popup_menu",
 			  G_CALLBACK (ephy_bookmarks_editor_show_popup_cb),
