@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-bookmarks-export.c 7858 2008-01-13 20:42:01Z cosimoc $
+ *  $Id: ephy-bookmarks-export.c 7869 2008-01-15 07:56:31Z xan $
  */
 
 #include "config.h"
@@ -395,7 +395,7 @@ ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 	char *tmp_file_path;
 	int ret;
 
-	LOG ("Exporting as RDF to %s", filename);
+	LOG ("Exporting as RDF to %s", file_path);
 
 	START_PROFILER ("Exporting as RDF")
 
