@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-open-tabs-action.c 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ephy-open-tabs-action.c 7593 2007-10-28 13:54:32Z chpe $
  */
 
 #include "config.h"
@@ -43,7 +43,7 @@ activate_cb (GtkAction *action,
 	EphyLink *link;
 	EphyNode *node;
 	GPtrArray *children;
-	EphyTab *tab = 0;
+	EphyEmbed *embed = NULL;
 	const char *url;
 	guint i;
 
@@ -56,8 +56,8 @@ activate_cb (GtkAction *action,
 		node = g_ptr_array_index (children, i);
 
 		url = ephy_node_get_property_string (node, EPHY_NODE_BMK_PROP_LOCATION);
-		tab = ephy_link_open (link, url, tab,
-				      EPHY_LINK_NEW_TAB | ephy_link_flags_from_current_event ());
+		embed = ephy_link_open (link, url, embed,
+				        EPHY_LINK_NEW_TAB | ephy_link_flags_from_current_event ());
 	}
 }
 
