@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-extension.c 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ephy-extension.c 7581 2007-10-28 13:03:54Z chpe $
  */
 
 #include "config.h"
@@ -70,23 +70,23 @@ ephy_extension_detach_window (EphyExtension *extension,
 void
 ephy_extension_attach_tab (EphyExtension *extension,
 			   EphyWindow *window,
-			   EphyTab *tab)
+			   EphyEmbed *embed)
 {
 	EphyExtensionIface *iface = EPHY_EXTENSION_GET_IFACE (extension);
 	if (iface->attach_tab)
 	{
-		iface->attach_tab (extension, window, tab);
+		iface->attach_tab (extension, window, embed);
 	}
 }
 
 void
 ephy_extension_detach_tab (EphyExtension *extension,
 			   EphyWindow *window,
-			   EphyTab *tab)
+			   EphyEmbed *embed)
 {
 	EphyExtensionIface *iface = EPHY_EXTENSION_GET_IFACE (extension);
 	if (iface->detach_tab)
 	{
-		iface->detach_tab (extension, window, tab);
+		iface->detach_tab (extension, window, embed);
 	}
 }
