@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  $Id: ephy-find-toolbar.c 7501 2007-09-27 17:18:41Z cosimoc $
+ *  $Id: ephy-find-toolbar.c 7659 2007-11-10 22:06:43Z cyrilbois $
  */
 
 #include "config.h"
@@ -28,7 +28,9 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <gtk/gtkarrow.h>
+#include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkentry.h>
@@ -54,7 +56,9 @@ struct _EphyFindToolbarPrivate
 	GtkToolItem *sep;
 	GtkToolItem *status_item;
 	GtkWidget *status_label;
+	GtkWidget *case_sensitive;
 	gulong set_focus_handler;
+	guint source_id;
 	guint find_again_source_id;
 	guint preedit_changed : 1;
 	guint prevent_activate : 1;
@@ -102,12 +106,30 @@ get_find (EphyFindToolbar *toolbar)
 	return priv->find;
 }
 
+static gboolean
+set_status_notfound_cb (EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv;
+	PangoFontDescription *pango_desc = NULL;
+	
+	priv = toolbar->priv;
+	
+	pango_desc = pango_font_description_new ();
+	gtk_widget_modify_font (priv->status_label, pango_desc);
+	pango_font_description_free (pango_desc);
+	
+	priv->source_id = 0;
+	
+	return FALSE;
+}
+
 static void
 set_status (EphyFindToolbar *toolbar,
 	    EphyEmbedFindResult result)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	char *text = NULL;
+	PangoFontDescription *pango_desc = NULL;
 
 	switch (result)
 	{
@@ -115,7 +137,17 @@ set_status (EphyFindToolbar *toolbar,
 			text = NULL;
 			break;
 		case EPHY_EMBED_FIND_NOTFOUND:
-			text = _("Not found");
+			{
+				text = _("Not found");
+
+				pango_desc = pango_font_description_new ();
+				pango_font_description_set_weight (pango_desc, PANGO_WEIGHT_BOLD);
+				gtk_widget_modify_font (priv->status_label, pango_desc);
+				pango_font_description_free (pango_desc);
+				
+				gtk_widget_error_bell (GTK_WIDGET (priv->window));
+				priv->source_id = g_timeout_add (500, (GSourceFunc) set_status_notfound_cb, toolbar);
+			}
 			break;
 		case EPHY_EMBED_FIND_FOUNDWRAPPED:
 			text = _("Wrapped");
@@ -193,18 +225,11 @@ entry_changed_cb (GtkEntry *entry,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	const char *text;
-	char *lowercase;
 	EphyEmbedFindResult result;
 	gboolean case_sensitive;
 
 	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
-
-	/* Search case-sensitively iff the string includes 
-	 * non-lowercase character.
-	 */
-	lowercase = g_utf8_strdown (text, -1);
-	case_sensitive = g_utf8_collate (text, lowercase) != 0;
-	g_free (lowercase);
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
 
 	ephy_embed_find_set_properties (get_find (toolbar), text, case_sensitive);
 	result = ephy_embed_find_find (get_find (toolbar), text, priv->links_only);
@@ -286,6 +311,71 @@ entry_activate_cb (GtkWidget *entry,
 	}
 }
 
+static void
+case_sensitive_menu_toggled_cb (GtkWidget *check,
+				EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv = toolbar->priv;
+	gboolean case_sensitive;
+
+	case_sensitive = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (check));
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->case_sensitive),
+				      case_sensitive);
+}
+
+static void
+case_sensitive_toggled_cb (GtkWidget *check,
+			   EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv = toolbar->priv;
+	const char *text;
+	gboolean case_sensitive;
+	GtkWidget *proxy;
+
+	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
+
+	proxy = gtk_tool_item_get_proxy_menu_item (GTK_TOOL_ITEM (gtk_widget_get_parent (check)),
+						   "menu-proxy");
+
+	if (proxy != NULL)
+	{
+		g_signal_handlers_block_by_func
+			(proxy, G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
+
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (proxy),
+						case_sensitive);
+		g_signal_handlers_unblock_by_func
+			(proxy, G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
+	}
+
+	ephy_embed_find_set_properties (get_find (toolbar), text, case_sensitive);
+}
+
+static gboolean
+toolitem_create_menu_proxy_cb (GtkToolItem *toolitem,
+			       EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv = toolbar->priv;
+	GtkWidget *checkbox_menu;
+	gboolean case_sensitive;
+
+	/* Create a menu item, and sync it */
+	checkbox_menu = gtk_check_menu_item_new_with_mnemonic (_("_Case sensitive"));
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (checkbox_menu),
+					case_sensitive);
+
+	g_signal_connect (checkbox_menu, "toggled",
+			  G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
+
+	gtk_tool_item_set_proxy_menu_item (toolitem, "menu-proxy",
+					   checkbox_menu);
+
+	return TRUE;
+}
+
 static void
 set_focus_cb (EphyWindow *window,
 	      GtkWidget *widget,
@@ -347,6 +437,7 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 	GtkToolbar *gtoolbar;
 	GtkToolItem *item;
 	GtkWidget *alignment, *arrow, *box;
+	GtkWidget *checkbox;
 
 	priv = toolbar->priv = EPHY_FIND_TOOLBAR_GET_PRIVATE (toolbar);
 	gtoolbar = GTK_TOOLBAR (toolbar);
@@ -378,9 +469,8 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 	arrow = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_NONE);
 	priv->prev = gtk_tool_button_new (arrow, _("Find Previous"));
 	gtk_tool_item_set_is_important (priv->prev, TRUE);
-	gtk_tool_item_set_tooltip (priv->prev, gtoolbar->tooltips,
-				   _("Find previous occurrence of the search string"),
-				   NULL);
+	gtk_tool_item_set_tooltip_text (priv->prev,
+					_("Find previous occurrence of the search string"));
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->prev, -1);
 	gtk_widget_show_all (GTK_WIDGET (priv->prev));
 
@@ -388,12 +478,23 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 	arrow = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
 	priv->next = gtk_tool_button_new (arrow, _("Find Next"));
 	gtk_tool_item_set_is_important (priv->next, TRUE);
-	gtk_tool_item_set_tooltip (priv->next, gtoolbar->tooltips,
-				   _("Find next occurrence of the search string"),
-				   NULL);
+	gtk_tool_item_set_tooltip_text (priv->next,
+					_("Find next occurrence of the search string"));
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->next, -1);
 	gtk_widget_show_all (GTK_WIDGET (priv->next));
 
+	/* Case sensitivity */
+	checkbox = gtk_check_button_new_with_mnemonic (_("_Case sensitive"));
+	priv->case_sensitive = checkbox;
+	item = gtk_tool_item_new ();
+	gtk_container_add (GTK_CONTAINER (item), checkbox);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+	gtk_widget_show_all (GTK_WIDGET (item));
+
+	/* Populate the overflow menu */
+	g_signal_connect (item, "create-menu-proxy",
+			  G_CALLBACK (toolitem_create_menu_proxy_cb), toolbar);
+
 	priv->sep = gtk_separator_tool_item_new ();
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->sep, -1);
 	
@@ -417,6 +518,8 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 				  G_CALLBACK (find_next_cb), toolbar);
 	g_signal_connect_swapped (priv->prev, "clicked",
 				  G_CALLBACK (find_prev_cb), toolbar);
+	g_signal_connect (priv->case_sensitive, "toggled",
+			  G_CALLBACK (case_sensitive_toggled_cb), toolbar);
 }
 
 static void
@@ -430,6 +533,12 @@ ephy_find_toolbar_dispose (GObject *object)
 		g_object_unref (priv->find);
 		priv->find = NULL;
 	}
+	
+	if (priv->source_id != 0)
+	{
+		g_source_remove (priv->source_id);
+		priv->source_id = 0;
+	}
 
 	if (priv->find_again_source_id != 0)
 	{
@@ -688,7 +797,6 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 			gboolean typing_ahead)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
-	gboolean clear_search = typing_ahead;
 
 	g_return_if_fail (priv->embed != NULL);
 
@@ -697,14 +805,7 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 
 	clear_status (toolbar);
 
-	if (clear_search)
-	{
-		gtk_entry_set_text (GTK_ENTRY (priv->entry), "");
-	}
-	else
-	{
-		gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
-	}
+	gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
 
 	gtk_widget_show (GTK_WIDGET (toolbar));
 	gtk_widget_grab_focus (GTK_WIDGET (toolbar));
