@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-history-window.c 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ephy-history-window.c 7791 2007-12-19 21:46:55Z cosimoc $
  */
 
 #include "config.h"
@@ -47,7 +47,6 @@
 #include <string.h>
 #include <time.h>
 
-#include "ephy-node-view.h"
 #include "ephy-window.h"
 #include "ephy-history-window.h"
 #include "ephy-shell.h"
@@ -69,6 +68,7 @@
 #include "ephy-bookmarks-ui.h"
 #include "ephy-prefs.h"
 #include "ephy-gui.h"
+#include "ephy-time-helpers.h"
 
 static const GtkTargetEntry page_drag_types [] =
 {
@@ -112,6 +112,11 @@ static void cmd_select_all		  (GtkAction *action,
 					   EphyHistoryWindow *editor);
 static void cmd_help_contents		  (GtkAction *action,
 					   EphyHistoryWindow *editor);
+static void cmd_view_columns		  (GtkAction *action,
+					   EphyHistoryWindow *view);
+static void search_entry_search_cb 	  (GtkWidget *entry,
+					   char *search_text,
+					   EphyHistoryWindow *editor);
 
 #define EPHY_HISTORY_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_HISTORY_WINDOW, EphyHistoryWindowPrivate))
 
@@ -135,6 +140,7 @@ struct _EphyHistoryWindowPrivate
 	EphyNode *selected_site;
 	GtkTreeViewColumn *title_col;
 	GtkTreeViewColumn *address_col;
+	GtkTreeViewColumn *datetime_col;
 };
 
 enum
@@ -175,9 +181,6 @@ static const GtkActionEntry ephy_history_ui_entries [] = {
 	{ "OpenInTab", STOCK_NEW_TAB, N_("Open in New _Tab"), "<shift><control>O",
 	  N_("Open the selected history link in a new tab"),
 	  G_CALLBACK (cmd_open_bookmarks_in_tabs) },
-	{ "Delete", GTK_STOCK_DELETE, N_("_Delete"), NULL,
-	  N_("Delete the selected history link"),
-	  G_CALLBACK (cmd_delete) },
 	{ "BookmarkLink", STOCK_ADD_BOOKMARK, N_("Add _Bookmark…"), "<control>D",
 	  N_("Bookmark the selected history link"),
 	  G_CALLBACK (cmd_bookmark_link) },
@@ -195,7 +198,10 @@ static const GtkActionEntry ephy_history_ui_entries [] = {
 	{ "Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V",
 	  N_("Paste the clipboard"),
 	  G_CALLBACK (cmd_paste) },
-	{ "SelectAll", NULL, N_("Select _All"), "<control>A",
+	{ "Delete", GTK_STOCK_DELETE, N_("_Delete"), "<control>T",
+	  N_("Delete the selected history link"),
+	  G_CALLBACK (cmd_delete) },
+	{ "SelectAll", GTK_STOCK_SELECT_ALL, N_("Select _All"), "<control>A",
 	  N_("Select all history links or text"),
 	  G_CALLBACK (cmd_select_all) },
 	{ "Clear", GTK_STOCK_CLEAR, N_("Clear _History"), NULL,
@@ -206,28 +212,27 @@ static const GtkActionEntry ephy_history_ui_entries [] = {
 	{ "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1",
 	  N_("Display history help"),
 	  G_CALLBACK (cmd_help_contents) },
-	{ "HelpAbout", GNOME_STOCK_ABOUT, N_("_About"), NULL,
+	{ "HelpAbout", GTK_STOCK_ABOUT, N_("_About"), NULL,
 	  N_("Display credits for the web browser creators"),
 	  G_CALLBACK (window_cmd_help_about) },
 };
 
-enum
+typedef enum
 {
-	VIEW_TITLE,
-	VIEW_ADDRESS,
-	VIEW_TITLE_AND_ADDRESS
-};
+	VIEW_TITLE = 1 << 0,
+	VIEW_ADDRESS = 1 << 1,
+	VIEW_DATETIME = 1 << 2
+} EphyHistoryWindowColumns;
 
-static const GtkRadioActionEntry ephy_history_radio_entries [] =
+static const GtkToggleActionEntry ephy_history_toggle_entries [] =
 {
 	/* View Menu */
 	{ "ViewTitle", NULL, N_("_Title"), NULL,
-	  N_("Show only the title column"), VIEW_TITLE },
+	  N_("Show the title column"), G_CALLBACK (cmd_view_columns), TRUE },
 	{ "ViewAddress", NULL, N_("_Address"), NULL,
-	  N_("Show only the address column"), VIEW_ADDRESS },
-	{ "ViewTitleAddress", NULL, N_("Title a_nd Address"), NULL,
-	  N_("Show both the title and address columns"),
-	  VIEW_TITLE_AND_ADDRESS } 
+	  N_("Show the address column"), G_CALLBACK (cmd_view_columns), TRUE },
+	{ "ViewDateTime", NULL, N_("_Date and Time"), NULL,
+	  N_("Show the date and time column"), G_CALLBACK (cmd_view_columns), TRUE }
 };
 
 static void
@@ -515,45 +520,70 @@ cmd_help_contents (GtkAction *action,
 }
 
 static void
-set_columns_visibility (EphyHistoryWindow *view, int value)
+set_column_visibility (EphyHistoryWindow *view,
+		       const char *action_name,
+		       gboolean active)
 {
-	switch (value)
+	if (strcmp (action_name, "ViewTitle") == 0)
 	{
-		case VIEW_TITLE:
-			gtk_tree_view_column_set_visible (view->priv->title_col, TRUE);
-			gtk_tree_view_column_set_visible (view->priv->address_col, FALSE);
-			break;
-		case VIEW_ADDRESS:
-			gtk_tree_view_column_set_visible (view->priv->title_col, FALSE);
-			gtk_tree_view_column_set_visible (view->priv->address_col, TRUE);
-			break;
-		case VIEW_TITLE_AND_ADDRESS:
-			gtk_tree_view_column_set_visible (view->priv->title_col, TRUE);
-			gtk_tree_view_column_set_visible (view->priv->address_col, TRUE);
-			break;
+		gtk_tree_view_column_set_visible (view->priv->title_col, active);
+	}
+	if (strcmp (action_name, "ViewAddress") == 0)
+	{
+		gtk_tree_view_column_set_visible (view->priv->address_col, active);
+	}
+	if (strcmp (action_name, "ViewDateTime") == 0)
+	{
+		gtk_tree_view_column_set_visible (view->priv->datetime_col, active);
 	}
 }
 
+static void
+set_all_columns_visibility (EphyHistoryWindow *view,
+			    EphyHistoryWindowColumns details_value)
+{
+	GtkActionGroup *action_group;
+	GtkAction *action_title, *action_address, *action_datetime;
+
+	action_group = view->priv->action_group;
+	action_title = gtk_action_group_get_action (action_group, "ViewTitle");
+	action_address = gtk_action_group_get_action (action_group, "ViewAddress");
+	action_datetime = gtk_action_group_get_action (action_group, "ViewDateTime");
+	
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_title), (details_value & VIEW_TITLE));
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_address), (details_value & VIEW_ADDRESS));
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_datetime), (details_value & VIEW_DATETIME));
+}
+
 static void
 cmd_view_columns (GtkAction *action,
-		  GtkRadioAction *current,
 		  EphyHistoryWindow *view)
 {
-	int value;
+	gboolean active;
+	const char *action_name;
 	GSList *svalues = NULL;
 
-	value = gtk_radio_action_get_current_value (current);
-	set_columns_visibility (view, value);
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	action_name = gtk_action_get_name (action);
+	set_column_visibility (view, action_name, active);
+	
+	svalues = eel_gconf_get_string_list (CONF_HISTORY_VIEW_DETAILS);
 
-	switch (value)
+	if (active)
 	{
-		case VIEW_TITLE:
-			svalues = g_slist_append (svalues, (gpointer)"title");
-			break;
-		case VIEW_TITLE_AND_ADDRESS:
-			svalues = g_slist_append (svalues, (gpointer)"title");
-			svalues = g_slist_append (svalues, (gpointer)"address");
-			break;
+		if (!g_slist_find_custom (svalues, (gpointer) action_name, (GCompareFunc) strcmp))
+		{
+			svalues = g_slist_append (svalues, (gpointer) action_name);
+		}
+	}
+	else
+	{
+		GSList *delete;
+		delete = g_slist_find_custom (svalues, (gpointer) action_name, (GCompareFunc) strcmp);
+		if (delete)
+		{
+			svalues = g_slist_delete_link (svalues, delete);
+		}
 	}
 
 	eel_gconf_set_string_list (CONF_HISTORY_VIEW_DETAILS, svalues);
@@ -662,6 +692,25 @@ ephy_history_window_node_activated_cb (GtkWidget *view,
 			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
+static void
+ephy_history_window_node_middle_clicked_cb (GtkWidget *view,
+					    EphyNode *node,
+					    EphyHistoryWindow *editor)
+{
+	EphyWindow *window;
+	const char *location;
+
+	window = EPHY_WINDOW (get_target_window (editor));
+
+	location = ephy_node_get_property_string
+		(node, EPHY_NODE_PAGE_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
+
+	ephy_shell_new_tab (ephy_shell, window, NULL, location,
+			    EPHY_NEW_TAB_OPEN_PAGE |
+			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+}
+
 static void
 ephy_history_window_update_menu (EphyHistoryWindow *editor)
 {
@@ -918,7 +967,8 @@ add_by_word_filter (EphyHistoryWindow *editor, EphyNodeFilter *filter, int level
 {
 	const char *search_text;
 
-	search_text = gtk_entry_get_text (GTK_ENTRY (editor->priv->search_entry));
+	search_text = gtk_entry_get_text (GTK_ENTRY (ephy_icon_entry_get_entry
+						     (EPHY_ICON_ENTRY (editor->priv->search_entry))));
 	if (search_text == NULL) return;
 
 	ephy_node_filter_add_expression
@@ -980,7 +1030,13 @@ site_node_selected_cb (EphyNodeView *view,
 	}
 	else
 	{
+		g_signal_handlers_block_by_func (EPHY_SEARCH_ENTRY (editor->priv->search_entry),
+						 G_CALLBACK (search_entry_search_cb),
+						 editor);
 		ephy_search_entry_clear (EPHY_SEARCH_ENTRY (editor->priv->search_entry));
+		g_signal_handlers_unblock_by_func (EPHY_SEARCH_ENTRY (editor->priv->search_entry),
+						   G_CALLBACK (search_entry_search_cb),
+						   editor);
 		setup_filters (editor, TRUE, FALSE);
 	}
 }
@@ -1012,11 +1068,31 @@ time_combo_changed_cb (GtkWidget *combo, EphyHistoryWindow *editor)
 	setup_filters (editor, TRUE, TRUE);
 }
 
+static gboolean
+search_entry_clear_cb (GtkWidget *ebox,
+		       GdkEventButton *event,
+		       GtkWidget *entry)
+{
+	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
+	
+	if (event->type == GDK_BUTTON_RELEASE && 
+	    event->button == 1 /* left */ && 
+	    state == 0)
+	{	
+		ephy_search_entry_clear (EPHY_SEARCH_ENTRY (entry));
+		
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+
 static GtkWidget *
 build_search_box (EphyHistoryWindow *editor)
 {
 	GtkWidget *box, *label, *entry;
 	GtkWidget *combo;
+	GtkWidget *cleaner, *ebox;
 	char *str;
 	int time_range;
 
@@ -1028,7 +1104,23 @@ build_search_box (EphyHistoryWindow *editor)
 	add_focus_monitor (editor, entry);
 	add_entry_monitor (editor, entry);
 	editor->priv->search_entry = entry;
-	gtk_widget_show (entry);
+    
+	cleaner = gtk_image_new_from_stock (GTK_STOCK_CLEAR,
+					    GTK_ICON_SIZE_MENU);
+	ebox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
+	
+	gtk_widget_add_events (ebox, GDK_BUTTON_PRESS_MASK |
+			       GDK_BUTTON_RELEASE_MASK);
+	g_signal_connect (ebox , "button-release-event",
+			  G_CALLBACK (search_entry_clear_cb), 
+			  entry);
+	gtk_widget_set_tooltip_text (ebox,
+			             _("Clear"));
+	gtk_container_add (GTK_CONTAINER (ebox), cleaner);
+	ephy_icon_entry_pack_widget ((EPHY_ICON_ENTRY (entry)), ebox, FALSE);
+    
+	gtk_widget_show_all (entry);
 
 	label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -1138,37 +1230,76 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	g_value_take_object (value, pixbuf);
 }
 
+static void
+convert_cell_data_func (GtkTreeViewColumn *column,
+			GtkCellRenderer *renderer,
+			GtkTreeModel *model,
+			GtkTreeIter *iter,
+			gpointer user_data)
+{
+	int col_id = (int) user_data;
+	int value;
+	time_t time;
+	char *friendly;
+	
+	gtk_tree_model_get (model, iter,
+			    col_id,	
+			    &value,
+			    -1);
+	time = (time_t) value;
+	
+	friendly = ephy_time_helpers_utf_friendly_time (time);
+	g_object_set (renderer, "text", friendly, NULL);
+	g_free (friendly);
+}
+
+static void
+parse_time_into_date (GtkTreeViewColumn *column,
+		      int column_id)
+{
+	GList *renderers_list;
+	GtkCellRenderer *renderer;
+	
+	renderers_list = gtk_tree_view_column_get_cell_renderers (column);
+	renderer = GTK_CELL_RENDERER (renderers_list->data);
+	gtk_tree_view_column_set_cell_data_func (column, renderer,
+						 (GtkTreeCellDataFunc) convert_cell_data_func,
+						 (gpointer) column_id,
+						 NULL);
+	g_list_free (renderers_list);
+}
 static void
 view_selection_changed_cb (GtkWidget *view, EphyHistoryWindow *editor)
 {
 	ephy_history_window_update_menu (editor);
 }
 
-static int
+static EphyHistoryWindowColumns
 get_details_value (void)
 {
-	int value;
+	guint value = 0;
 	GSList *svalues;
 
 	svalues = eel_gconf_get_string_list (CONF_HISTORY_VIEW_DETAILS);
-	if (svalues == NULL) return VIEW_TITLE_AND_ADDRESS;
+	if (svalues == NULL) 
+	{
+		svalues = g_slist_append (svalues, (gpointer) "ViewAddress");
+		svalues = g_slist_append (svalues, (gpointer) "ViewTitle");
+		eel_gconf_set_string_list (CONF_HISTORY_VIEW_DETAILS, svalues);
+		return (VIEW_ADDRESS | VIEW_TITLE);
+	}
 
-	if (g_slist_find_custom (svalues, "title", (GCompareFunc)strcmp) &&
-	    g_slist_find_custom (svalues, "address", (GCompareFunc)strcmp))
+	if (g_slist_find_custom (svalues, "ViewTitle", (GCompareFunc)strcmp))
 	{
-		value = VIEW_TITLE_AND_ADDRESS;
+		value |= VIEW_TITLE;
 	}
-	else if (g_slist_find_custom (svalues, "title", (GCompareFunc)strcmp))
+	if (g_slist_find_custom (svalues, "ViewAddress", (GCompareFunc)strcmp))
 	{
-		value = VIEW_TITLE;
+		value |= VIEW_ADDRESS;
 	}
-	else if (g_slist_find_custom (svalues, "address", (GCompareFunc)strcmp))
+	if (g_slist_find_custom (svalues, "ViewDateTime", (GCompareFunc)strcmp))
 	{
-		value = VIEW_ADDRESS;
-	}
-	else
-	{
-		value = VIEW_TITLE_AND_ADDRESS;
+		value |= VIEW_DATETIME;
 	}
 
 	g_slist_foreach (svalues, (GFunc) g_free, NULL);
@@ -1188,7 +1319,8 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	EphyNode *node;
 	GtkUIManager *ui_merge;
 	GtkActionGroup *action_group;
-	int url_col_id, title_col_id, details_value;
+	int url_col_id, title_col_id, datetime_col_id;
+	EphyHistoryWindowColumns details_value;
 
 	ephy_gui_ensure_window_group (GTK_WINDOW (editor));
 
@@ -1212,12 +1344,10 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 				      G_N_ELEMENTS (ephy_history_ui_entries), editor);
 
 	details_value = get_details_value ();
-	gtk_action_group_add_radio_actions (action_group,
-					    ephy_history_radio_entries,
-					    G_N_ELEMENTS (ephy_history_radio_entries),
-					    details_value,
-					    G_CALLBACK (cmd_view_columns), 
-					    editor);
+	gtk_action_group_add_toggle_actions (action_group,
+					     ephy_history_toggle_entries,
+					     G_N_ELEMENTS (ephy_history_toggle_entries),
+					     editor);
 
 	gtk_ui_manager_insert_action_group (ui_merge,
 					    action_group, 0);
@@ -1245,7 +1375,7 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 					"vscrollbar_policy", GTK_POLICY_AUTOMATIC,
 					"shadow_type", GTK_SHADOW_IN,
 					NULL);
-	gtk_paned_add1 (GTK_PANED (hpaned), scrolled_window);
+	gtk_paned_pack1 (GTK_PANED (hpaned), scrolled_window, TRUE, FALSE);
 	gtk_widget_show (scrolled_window);
 	editor->priv->sites_filter = ephy_node_filter_new ();
 	sites_view = ephy_node_view_new (node, editor->priv->sites_filter);
@@ -1294,7 +1424,7 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			  editor);
 
 	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_paned_add2 (GTK_PANED (hpaned), vbox);
+	gtk_paned_pack2 (GTK_PANED (hpaned), vbox, TRUE, FALSE);
 	gtk_widget_show (vbox);
 
 	gtk_box_pack_start (GTK_BOX (vbox),
@@ -1320,14 +1450,26 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	title_col_id = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Title"),
 				                  G_TYPE_STRING, EPHY_NODE_PAGE_PROP_TITLE,
 				                  EPHY_NODE_VIEW_SORTABLE |
-					          EPHY_NODE_VIEW_SEARCHABLE, NULL, &col);
-	gtk_tree_view_column_set_max_width (col, 250);
+					          EPHY_NODE_VIEW_SEARCHABLE |
+						  EPHY_NODE_VIEW_ELLIPSIZED, NULL, &col);
+	gtk_tree_view_column_set_min_width (col, 300);
+	gtk_tree_view_column_set_resizable (col, TRUE);
 	editor->priv->title_col = col;
+	
 	url_col_id = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Address"),
 				                G_TYPE_STRING, EPHY_NODE_PAGE_PROP_LOCATION,
-				                EPHY_NODE_VIEW_SORTABLE, NULL, &col);
-	gtk_tree_view_column_set_max_width (col, 200);
+				                EPHY_NODE_VIEW_SORTABLE |
+						EPHY_NODE_VIEW_ELLIPSIZED, NULL, &col);
+	gtk_tree_view_column_set_min_width (col, 300);
+	gtk_tree_view_column_set_resizable (col, TRUE);
 	editor->priv->address_col = col;
+
+	datetime_col_id = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Date"),
+						     G_TYPE_INT, EPHY_NODE_PAGE_PROP_LAST_VISIT,
+						     EPHY_NODE_VIEW_SORTABLE, NULL, &col);
+	editor->priv->datetime_col = col;
+	parse_time_into_date (editor->priv->datetime_col, datetime_col_id);
+
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (pages_view),
 					   page_drag_types,
 				           G_N_ELEMENTS (page_drag_types),
@@ -1343,6 +1485,10 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			  "node_activated",
 			  G_CALLBACK (ephy_history_window_node_activated_cb),
 			  editor);
+	g_signal_connect (G_OBJECT (pages_view),
+			  "node-middle-clicked",
+			  G_CALLBACK (ephy_history_window_node_middle_clicked_cb),
+			  editor);
 	g_signal_connect (G_OBJECT (pages_view),
 			  "popup_menu",
 			  G_CALLBACK (ephy_history_window_show_popup_cb),
@@ -1364,7 +1510,7 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			       "history_paned",
 		               130);
 
-	set_columns_visibility (editor, details_value);
+	set_all_columns_visibility (editor, details_value);
 	setup_filters (editor, TRUE, TRUE);
 }
 
