@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-link.c 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ephy-link.c 7606 2007-10-28 14:16:50Z chpe $
  */
 
 #include "config.h"
@@ -48,12 +48,12 @@ ephy_link_base_init (gpointer g_class)
 			 EPHY_TYPE_LINK,
 			 G_SIGNAL_RUN_LAST,
 			 G_STRUCT_OFFSET (EphyLinkIface, open_link),
-			 ephy_signal_accumulator_object, ephy_tab_get_type,
+			 ephy_signal_accumulator_object, ephy_embed_get_type,
 			 ephy_marshal_OBJECT__STRING_OBJECT_FLAGS,
-			 EPHY_TYPE_TAB,
+			 GTK_TYPE_WIDGET /* Can't use an interface type here */,
 			 3,
 			 G_TYPE_STRING,
-			 EPHY_TYPE_TAB,
+			 GTK_TYPE_WIDGET /* Can't use an interface type here */,
 			 EPHY_TYPE_LINK_FLAGS);
 
 		initialised = TRUE;
@@ -82,21 +82,21 @@ ephy_link_get_type (void)
 	return type;
 }
 
-EphyTab	*
+EphyEmbed *
 ephy_link_open (EphyLink *link,
 		const char *address,
-		EphyTab *tab,
+		EphyEmbed *embed,
 		EphyLinkFlags flags)
 {
-	EphyTab *new_tab = NULL;
+	EphyEmbed *new_embed = NULL;
 
-	LOG ("ephy_link_open address \"%s\" parent-tab %p flags %u", address, tab, flags);
+	LOG ("ephy_link_open address \"%s\" parent-embed %p flags %u", address, embed, flags);
 
 	g_signal_emit (link, signals[OPEN_LINK], 0,
-		       address, tab, flags,
-		       &new_tab);
+		       address, embed, flags,
+		       &new_embed);
 
-	return new_tab;
+	return new_embed;
 }
 
 EphyLinkFlags
