@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2003, 2004 Marco Pesenti Gritti
  *  Copyright © 2003, 2004 Christian Persch
@@ -16,11 +17,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-location-action.c 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ephy-location-action.c 7828 2007-12-26 19:00:00Z diegoe $
  */
 
 #include "config.h"
 
+#include "ephy-embed-container.h"
+#include "ephy-history.h"
 #include "ephy-location-action.h"
 #include "ephy-location-entry.h"
 #include "ephy-shell.h"
@@ -34,7 +37,10 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtkentrycompletion.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtktreemodelsort.h>
 
+
+#include <string.h>
 #define EPHY_LOCATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_LOCATION_ACTION, EphyLocationActionPrivate))
 
 struct _EphyLocationActionPrivate
@@ -45,6 +51,7 @@ struct _EphyLocationActionPrivate
 	char *typed_address;
 	EphyNode *smart_bmks;
 	EphyBookmarks *bookmarks;
+	EphyNode *history;
 	GdkPixbuf *icon;
 	char *lock_stock_id;
 	char *lock_tooltip;
@@ -84,6 +91,135 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 static GObjectClass *parent_class = NULL;
 
+static const struct
+{
+	const char *prefix;
+	int len;
+}
+web_prefixes [] =
+{
+	{ "http://www.", 11 },
+	{ "http://ftp.", 11 },
+	{ "http://", 7 },
+	{ "https://www.", 12 },
+	{ "https://", 8 },
+	{ "ftp://", 6},
+	{ "ftp://ftp.", 10},
+	{ "www.", 4 },
+	{ "ftp.", 4}
+};
+
+
+static gboolean
+keyword_match (const char *list,
+	       const char *keyword)
+{
+	const char *p;
+	gsize keyword_len;
+
+	p = list;
+	keyword_len = strlen (keyword);
+
+	while (*p)
+	{
+		int i;
+
+		for (i = 0; i < keyword_len; i++)
+		{
+			if (p[i] != keyword[i])
+			{
+				goto next_token;
+			}
+		}
+	  
+		return TRUE;
+	  
+		next_token:
+
+		while (*p && !g_ascii_ispunct(*p) && !g_ascii_isspace(*p)) p++;
+		while (*p && (g_ascii_ispunct(*p) || g_ascii_isspace(*p))) p++;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+completion_func (GtkEntryCompletion *completion,
+		 const char *key,
+		 GtkTreeIter *iter,
+		 gpointer data)
+{
+	int i, len_key, len_prefix;
+	char *item = NULL;
+	char *url = NULL;
+	char *keywords = NULL;
+	gboolean ret = FALSE;
+	GtkTreeModel *model;
+	GtkTreeIter iter2;
+	EphyLocationActionPrivate *priv;
+	
+	priv = EPHY_LOCATION_ACTION (data)->priv;
+
+	model = gtk_entry_completion_get_model (completion);
+
+	gtk_tree_model_get (model, iter,
+			    EPHY_COMPLETION_TEXT_COL, &item,
+			    EPHY_COMPLETION_URL_COL, &url,
+			    EPHY_COMPLETION_KEYWORDS_COL, &keywords,
+			    -1);
+
+	len_key = strlen (key);
+	if (!strncasecmp (key, item, len_key))
+	{
+		ret = TRUE;
+	}
+	else if (keyword_match (keywords, key))
+	{
+		ret = TRUE;
+	}
+	else if (!strncasecmp (key, url, len_key))
+	{
+		ret = TRUE;
+	}
+	else
+	{
+		for (i = 0; i < G_N_ELEMENTS (web_prefixes); i++)
+		{
+			len_prefix = web_prefixes[i].len;
+			if (!strncmp (web_prefixes[i].prefix, item, len_prefix) &&
+			    !strncasecmp (key, item + len_prefix, len_key))
+			{
+				ret = TRUE;
+				break;
+			}
+			else if (!strncmp (web_prefixes[i].prefix, url, len_prefix) &&
+			        !strncasecmp (key, url + len_prefix, len_key))
+			{
+				ret = TRUE;
+				break;
+			}
+
+		}
+	}
+	
+	gtk_tree_model_sort_convert_iter_to_child_iter 
+		(GTK_TREE_MODEL_SORT (model), &iter2, iter);
+
+	/* This checks if the item's URL exists as a bookmark, if that's true 
+	 * and we are seeing an history item, then we skip showing it.
+	 * The bookmark will be shown instead since we are not filtering it out.
+	 */
+	if (ephy_bookmarks_find_bookmark (priv->bookmarks, url) != NULL &&
+	    (iter2.user_data2 == priv->history))
+		ret = FALSE;
+
+	g_free (item);
+	g_free (url);
+	g_free (keywords);
+
+	return ret;
+}
+
 GType
 ephy_location_action_get_type (void)
 {
@@ -283,23 +419,24 @@ get_location_cb (EphyLocationEntry *entry,
 		EphyLocationAction *action)
 {
 	EphyLocationActionPrivate *priv = action->priv;
-	EphyTab *tab;
+	EphyEmbed *embed;
 	
-	tab = ephy_window_get_active_tab (priv->window);
-	g_return_val_if_fail (tab != NULL, NULL);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (priv->window));
 
-	return g_strdup (ephy_tab_get_address (tab));
+	return g_strdup (ephy_embed_get_address (embed));
 }
 
 static char *
 get_title_cb (EphyLocationEntry *entry,
 	      EphyLocationAction *action)
 {
-	EphyTab *tab;
+	EphyEmbed *embed;
 
-	tab = ephy_window_get_active_tab (action->priv->window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (action->priv->window));
 
-	return g_strdup (ephy_tab_get_title (tab));
+	return g_strdup (ephy_embed_get_title (embed));
 }
 
 static void
@@ -370,7 +507,12 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 						    EPHY_COMPLETION_KEYWORDS_COL,
 						    EPHY_COMPLETION_RELEVANCE_COL,
 						    EPHY_COMPLETION_EXTRA_COL,
-						    EPHY_COMPLETION_FAVICON_COL);
+						    EPHY_COMPLETION_FAVICON_COL,
+						    EPHY_COMPLETION_URL_COL);
+
+		ephy_location_entry_set_completion_func (EPHY_LOCATION_ENTRY (proxy), 
+							completion_func, 
+							EPHY_LOCATION_ACTION (action));
 
 		add_completion_actions (action, proxy);
 
@@ -727,6 +869,9 @@ ephy_location_action_init (EphyLocationAction *action)
 		(action->priv->bookmarks);
 
 	init_actions_list (action);
+	
+	priv->history = ephy_history_get_pages (EPHY_HISTORY (
+			ephy_embed_shell_get_global_history (embed_shell)));
 
 	ephy_node_signal_connect_object (priv->smart_bmks,
 			                 EPHY_NODE_CHILD_ADDED,
