@@ -15,13 +15,14 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-toolbar.c 6772 2006-12-30 22:15:15Z chpe $
+ *  $Id: ephy-toolbar.c 7695 2007-11-16 15:03:10Z xan $
  */
 
 #include "config.h"
 
+#include "ephy-embed-container.h"
 #include "ephy-toolbar.h"
 #include "ephy-link.h"
 #include "ephy-go-action.h"
@@ -168,21 +169,21 @@ sync_user_input_cb (EphyLocationAction *action,
 		    EphyToolbar *toolbar)
 {
 	EphyToolbarPrivate *priv = toolbar->priv;
-	EphyTab *tab;
+	EphyEmbed *embed;
 	const char *address;
 
 	LOG ("sync_user_input_cb");
 
 	if (priv->updating_address) return;
 
-	tab = ephy_window_get_active_tab (priv->window);
-	g_return_if_fail (EPHY_IS_TAB (tab));
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (priv->window));
+        g_assert (EPHY_IS_EMBED (embed));
 
 	address = ephy_location_action_get_address (action);
 
 	priv->updating_address = TRUE;
-	ephy_tab_set_typed_address (tab, address,
-				    EPHY_TAB_ADDRESS_EXPIRE_CURRENT);
+	ephy_embed_set_typed_address (embed, address,
+                                      EPHY_EMBED_ADDRESS_EXPIRE_CURRENT);
 	priv->updating_address = FALSE;
 }
 
@@ -456,6 +457,26 @@ ephy_toolbar_set_navigation_actions (EphyToolbar *toolbar,
 	ephy_action_change_sensitivity_flags (priv->actions[UP_ACTION], SENS_FLAG, !up);
 }
 
+void
+ephy_toolbar_set_navigation_tooltips (EphyToolbar *toolbar,
+				      const char *back_title,
+				      const char *forward_title)
+{
+	EphyToolbarPrivate *priv = toolbar->priv;
+	GValue value = { 0 };
+
+	g_value_init (&value, G_TYPE_STRING);
+
+	g_value_set_static_string (&value, back_title);
+	g_object_set_property (G_OBJECT (priv->actions[BACK_ACTION]),
+			       "tooltip", &value);
+
+	g_value_set_static_string (&value, forward_title);
+	g_object_set_property (G_OBJECT (priv->actions[FORWARD_ACTION]),
+			       "tooltip", &value);
+	g_value_unset (&value);
+}
+
 void
 ephy_toolbar_set_security_state (EphyToolbar *toolbar,
 				 gboolean is_secure,
