@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2002 Marco Pesenti Gritti
  *
@@ -15,12 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ppview-toolbar.c 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ppview-toolbar.c 7709 2007-11-16 23:32:41Z xan $
  */
 
 #include "config.h"
 
 #include "ppview-toolbar.h"
+#include "ephy-embed-container.h"
 #include "ephy-window.h"
 
 #include <string.h>
@@ -28,6 +30,7 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtktoolbar.h>
 #include <gtk/gtkuimanager.h>
 
 static void ppview_toolbar_class_init (PPViewToolbarClass *klass);
@@ -205,7 +208,8 @@ toolbar_update_sensitivity (PPViewToolbar *t)
 	GtkAction *action;
 	GtkActionGroup *action_group = t->priv->action_group;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
 	pages = ephy_embed_print_preview_n_pages (embed);
@@ -224,6 +228,10 @@ toolbar_update_sensitivity (PPViewToolbar *t)
 static void
 ppview_toolbar_set_window (PPViewToolbar *t, EphyWindow *window)
 {
+	GtkAction *action;
+	GtkWidget *widget;
+	GtkToolbarStyle style;
+
 	g_return_if_fail (t->priv->window == NULL);
 
 	t->priv->window = window;
@@ -238,6 +246,25 @@ ppview_toolbar_set_window (PPViewToolbar *t, EphyWindow *window)
 	t->priv->ui_id = gtk_ui_manager_add_ui_from_string
 		(t->priv->manager, ui_info, -1, NULL);
 
+	action = gtk_action_group_get_action (t->priv->action_group,
+					      "PPVClose");
+	g_object_set (action,
+		      "short-label",
+		      _("Close print preview"),
+		      "is-important",
+		      TRUE,
+		      NULL);
+
+	/* Force the display of the text only when needed */
+	widget = gtk_ui_manager_get_widget (t->priv->manager,
+					    "/PPViewToolbar");
+	style = gtk_toolbar_get_style (GTK_TOOLBAR (widget));
+	if (style == GTK_TOOLBAR_ICONS)
+	{
+		gtk_toolbar_set_style (GTK_TOOLBAR (widget),
+				       GTK_TOOLBAR_BOTH_HORIZ);
+	}
+
 	toolbar_update_sensitivity (t);
 }
 
@@ -277,7 +304,7 @@ toolbar_cmd_ppv_goto_first (GtkUIManager *merge,
 	EphyWindow *window = t->priv->window;
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	ephy_embed_print_preview_navigate (embed, EPHY_EMBED_PRINTPREVIEW_HOME, 0);
@@ -294,7 +321,7 @@ toolbar_cmd_ppv_goto_last  (GtkUIManager *merge,
 	EphyWindow *window = t->priv->window;
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
 	ephy_embed_print_preview_navigate (embed,
@@ -313,7 +340,8 @@ clamp_page_limits (PPViewToolbar *t, int page)
 	EphyEmbed *embed;
 	int pages;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_val_if_fail (embed != NULL, -1);
 
 	pages = ephy_embed_print_preview_n_pages (embed);
@@ -325,10 +353,10 @@ static void
 toolbar_cmd_ppv_go_back  (GtkUIManager *merge,
 			  PPViewToolbar *t)
 {
-	EphyWindow *window = t->priv->window;
+        EphyEmbedContainer *window = EPHY_EMBED_CONTAINER (t->priv->window);
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child (window);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	t->priv->current_page = clamp_page_limits (t, t->priv->current_page - 1);
@@ -347,7 +375,7 @@ toolbar_cmd_ppv_go_forward (GtkUIManager *merge,
 	EphyWindow *window = t->priv->window;
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	t->priv->current_page = clamp_page_limits (t, t->priv->current_page + 1);
@@ -371,10 +399,11 @@ toolbar_cmd_ppv_close (GtkUIManager *merge,
 	window = t->priv->window;
 	g_return_if_fail (EPHY_IS_WINDOW (window));
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	ephy_window_set_print_preview (window, FALSE);
+	_ephy_window_set_print_preview (window, FALSE);
 
 	ephy_embed_set_print_preview_mode (embed, FALSE);
 }
