@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2000-2004 Marco Pesenti Gritti
  *
@@ -15,11 +16,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: window-commands.c 7522 2007-10-07 16:59:18Z diegoe $
+ *  $Id: window-commands.c 7858 2008-01-13 20:42:01Z cosimoc $
  */
 
 #include "config.h"
 
+#include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
 #include "ephy-shell.h"
@@ -47,14 +49,13 @@
 #include "ephy-bookmarks-ui.h"
 #include "ephy-link.h"
 #include "ephy-stock-icons.h"
+#include "ephy-string.h"
+#include "eel-app-launch-context.h"
 #include "pdm-dialog.h"
 
 #include <string.h>
 #include <glib.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomeui/gnome-stock-icons.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <gio/gio.h>
 #include <gtk/gtkaboutdialog.h>
 #include <gtk/gtkeditable.h>
 #include <gtk/gtkmain.h>
@@ -94,11 +95,12 @@ window_cmd_file_print_preview (GtkAction *action,
 {
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	ephy_embed_set_print_preview_mode (embed, TRUE);
-	ephy_window_set_print_preview (window, TRUE);
+	_ephy_window_set_print_preview (window, TRUE);
 }
 
 void
@@ -107,7 +109,8 @@ window_cmd_file_print (GtkAction *action,
 {
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	ephy_embed_print (embed);
@@ -117,28 +120,39 @@ void
 window_cmd_file_send_to	(GtkAction *action,
 			 EphyWindow *window)
 {
-	EphyTab *tab;
 	EphyEmbed *embed;
-	char *url, *location, *title;
+	char *handler, *command;
+	const char *location, *title;
+	GAppInfo *appinfo;
+	
+	if (eel_gconf_get_boolean ("/desktop/gnome/url-handlers/mailto/enabled") == FALSE)
+	{
+		/* FIXME: add some UI to inform the user? */
+		return;
+	}
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (tab != NULL);
-
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	location = gnome_vfs_escape_string (ephy_tab_get_address (tab));
-	title = gnome_vfs_escape_string (ephy_tab_get_title (tab));
+	location = ephy_embed_get_address (embed);
+	title = ephy_embed_get_title (embed);
 
-	url = g_strconcat ("mailto:",
-                           "?Subject=", title,
-                           "&Body=", location, NULL);
+	/* FIXME: better use g_app_info_get_default_for_uri_scheme () when it is
+	 * implemented.
+	 */
+	handler = eel_gconf_get_string ("/desktop/gnome/url-handlers/mailto/command");
+	command = g_strconcat (handler, "mailto:",
+			       "?Subject=\"", title,
+			       "\"&Body=\"", location, "\"", NULL);
+	
+	appinfo = g_app_info_create_from_commandline (command, NULL, 0, NULL);
+	ephy_file_launch_application (appinfo, NULL,
+				      gtk_get_current_event_time (),
+				      GTK_WIDGET (window));
 
-	gnome_vfs_url_show (url);
-
-	g_free (title);
-	g_free (location);
-	g_free (url);
+	g_free (handler);
+	g_free (command);
 }
 
 static gboolean
@@ -181,7 +195,8 @@ window_cmd_view_stop (GtkAction *action,
 {
 	EphyEmbed *embed;
 	
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
 	gtk_widget_grab_focus (GTK_WIDGET (embed));
@@ -195,7 +210,8 @@ window_cmd_view_reload (GtkAction *action,
 {
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
 	gtk_widget_grab_focus (GTK_WIDGET (embed));
@@ -219,18 +235,15 @@ void
 window_cmd_file_bookmark_page (GtkAction *action,
 			       EphyWindow *window)
 {
-	EphyTab *tab;
 	EphyEmbed *embed;
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (tab != NULL);
-
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
 	ephy_bookmarks_ui_add_bookmark (GTK_WINDOW (window),
-					ephy_tab_get_address (tab),
-					ephy_tab_get_title (tab));
+					ephy_embed_get_address (embed),
+					ephy_embed_get_title (embed));
 }
 
 static void
@@ -283,7 +296,7 @@ window_cmd_file_save_as (GtkAction *action,
 	EphyEmbed *embed;
 	EphyEmbedPersist *persist;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
 	persist = EPHY_EMBED_PERSIST
@@ -320,7 +333,7 @@ window_cmd_file_close_window (GtkAction *action,
 		              EphyWindow *window)
 {
 	GtkWidget *notebook;
-	EphyTab *tab;
+	EphyEmbed *embed;
 
 	notebook = ephy_window_get_notebook (window);
 
@@ -330,10 +343,10 @@ window_cmd_file_close_window (GtkAction *action,
 		return;
 	}
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (tab != NULL);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+	g_return_if_fail (embed != NULL);
 
-	g_signal_emit_by_name (notebook, "tab-close-request", tab);
+	g_signal_emit_by_name (notebook, "tab-close-request", embed);
 }
 
 void
@@ -383,7 +396,7 @@ window_cmd_edit_cut (GtkAction *action,
 	else
 	{
 		EphyEmbed *embed;
-		embed = ephy_window_get_active_embed (window);
+		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
 		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
@@ -405,7 +418,7 @@ window_cmd_edit_copy (GtkAction *action,
 	{
 		EphyEmbed *embed;
 
-		embed = ephy_window_get_active_embed (window);
+		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
 		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
@@ -427,7 +440,7 @@ window_cmd_edit_paste (GtkAction *action,
 	{
 		EphyEmbed *embed;
 
-		embed = ephy_window_get_active_embed (window);
+		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
 		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
@@ -449,7 +462,7 @@ window_cmd_edit_delete (GtkAction *action,
 	{
 		EphyEmbed *embed;
 
-		embed = ephy_window_get_active_embed (window);
+		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
 		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
@@ -471,7 +484,8 @@ window_cmd_edit_select_all (GtkAction *action,
 	{
 		EphyEmbed *embed;
 
-		embed = ephy_window_get_active_embed (window);
+		embed = ephy_embed_container_get_active_child 
+                  (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
 		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
@@ -518,6 +532,17 @@ window_cmd_view_fullscreen (GtkAction *action,
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
 	{
+		GtkWidget *toolbar_editor;
+		toolbar_editor = GTK_WIDGET (g_object_get_data (G_OBJECT (window),
+								"EphyToolbarEditor"));
+		if (toolbar_editor != NULL)
+		{
+			/* We don't want the toolbar editor to show
+			 * while in fullscreen.
+			 */
+			gtk_dialog_response (GTK_DIALOG (toolbar_editor),
+					     GTK_RESPONSE_DELETE_EVENT);
+		}
 		gtk_window_fullscreen (GTK_WINDOW (window));
 	}
 	else
@@ -552,14 +577,17 @@ save_source_completed_cb (EphyEmbedPersist *persist)
 {
 	const char *dest;
 	guint32 user_time;
+	GFile *file;
 
 	user_time = ephy_embed_persist_get_user_time (persist);
 	dest = ephy_embed_persist_get_dest (persist);
 	g_return_if_fail (dest != NULL);
 
-	ephy_file_delete_on_exit (dest);
+	file = g_file_new_for_path (dest);
+	ephy_file_delete_on_exit (file);
 
-	ephy_file_launch_handler ("text/plain", dest, user_time);
+	ephy_file_launch_handler ("text/plain", file, user_time);
+	g_object_unref (file);
 }
 
 static void
@@ -606,23 +634,25 @@ void
 window_cmd_view_page_source (GtkAction *action,
 			     EphyWindow *window)
 {
-	EphyTab *tab;
 	EphyEmbed *embed;
 	const char *address;
 	guint32 user_time;
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (tab != NULL);
-
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	address = ephy_tab_get_address (tab);
+	address = ephy_embed_get_address (embed);
 	user_time = gtk_get_current_event_time ();
 
 	if (g_str_has_prefix (address, "file://"))
 	{
-		ephy_file_launch_handler ("text/plain", address, user_time);
+		GFile *file;
+		
+		file = g_file_new_for_uri (address);
+		ephy_file_launch_handler ("text/plain", file, user_time);
+		
+		g_object_unref (file);
 	}
 	else
 	{
@@ -636,7 +666,8 @@ window_cmd_view_page_security_info (GtkAction *action,
 {
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	ephy_embed_show_page_certificate (embed);
@@ -659,24 +690,22 @@ window_cmd_edit_personal_data (GtkAction *action,
 		               EphyWindow *window)
 {
 	PdmDialog *dialog;
-	EphyTab *tab;
-	GnomeVFSURI *uri;
-	const char *host;
+	EphyEmbed *embed;
+	const char *address;
+	char *host;
 
-	tab = ephy_window_get_active_tab (window);
-	if (tab == NULL) return;
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (window));
+	if (embed == NULL) return;
 
-	uri = gnome_vfs_uri_new (ephy_tab_get_address (tab));
-
-	host = uri != NULL ? gnome_vfs_uri_get_host_name (uri) : NULL;
+	address = ephy_embed_get_address (embed);
+	
+	host = address != NULL ? ephy_string_get_host_name (address) : NULL;
 
 	dialog = EPHY_PDM_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell));
 	pdm_dialog_open (dialog, host);
 
-	if (uri != NULL)
-	{
-		gnome_vfs_uri_unref (uri);
-	}
+	g_free (host);
 }
 
 #ifdef ENABLE_CERTIFICATE_MANAGER
@@ -824,8 +853,8 @@ window_cmd_help_about (GtkAction *action,
 	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (shell));
 
 	comments = g_strdup_printf (_("Lets you view web pages and find information on the internet.\n"
-				      "Powered by Gecko %s"),
-				      ephy_embed_single_get_backend_name (single) + strlen ("gecko-"));
+				      "Powered by %s"), ephy_embed_single_get_backend_name (single) );
+
 	licence = g_strjoin ("\n\n",
 			     _(licence_part[0]),
 			     _(licence_part[1]),
@@ -955,7 +984,7 @@ window_cmd_load_location (GtkAction *action,
 		g_return_if_fail (address != NULL);
 
 		ephy_link_open (EPHY_LINK (window), address,
-			        ephy_window_get_active_tab (window),
+			        ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window)),
 				ephy_link_flags_from_current_event () | EPHY_LINK_ALLOW_FIXUP);
 	}
 }
